#!/usr/bin/env python

"""
Classes used to help with the generation of raw image data.

Copyright (C) 2009, 2010, 2011, 2012 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

class RawObject:

    "The basis for serialised program objects."

    def __init__(self, item):
        self.item = item

    def set_location(self, location, objtable, with_builtins):
        self.item.location = location
        return location + 1

    def finalise_location(self, with_builtins):
        pass

    def finalise_body_location(self, with_builtins):
        pass

    def is_generated(self, with_builtins):
        return with_builtins or self.item.module.name not in ("__builtins__", "native") or self.item.astnode.doc is not None

class UntranslatableInstruction(Exception):
    pass

class RSVPTranslator:

    "A class which provides the basis for the translation of RSVP instructions."

    def get_method(self, instruction):

        "Return the handler method for the given 'instruction'."

        instruction_name = instruction.__class__.__name__
        method = getattr(self, instruction_name, None)
        if method is None:
            raise UntranslatableInstruction, instruction_name
        return method

    def perform(self, instruction):

        "Perform the 'instruction', returning the next PC value or None."

        self.get_method(instruction)()

    def translate(self, code):

        "Translate the given 'code'."

        self.output_code = []

        for instruction in code:
            self.perform(instruction)

    def new_op(self, op):
        self.output_code.append(op)

# vim: tabstop=4 expandtab shiftwidth=4
